package taucs;

import java.util.*;

public class Taucs {	

// has to be called in the beginning before starting to work with the library
static public native void InitTaucsInterface();

// initializes a new matrix; returns the matrix ID
static public native int  CreateMatrix(int numRows, int numCols);
// releases the matrix 
static public native void ReleaseMatrix(int id);

// has to be called in the end after finishing working with the library
static public native void DeinitTaucsInterface();

// adds a new entry to the sparse matrix A
// i and j are 0-based
// if the A^T A matrix had been factored, this will destroy the factor
static public native void SetMatrixEntry(int id, int i, int j, double v);
static public native void AddToMatrixEntry(int id, int i, int j, double v);

// explicitly states that the matrix is spd or not
static public native boolean SetSPD(int id, boolean isSPD);

// Will create A^T A and its factorization, as well as store A^T
// returns true on success, false otherwise
static public native boolean FactorATA(int id);

// Solves the normal equations for Ax = b, namely A^T A x = A^T b
// it is assumed that enough space is allocated in x
// If a factorization of A^T A exists, it will be used, otherwise it will be created
// numRhs should be the the number of right-hand sides stored in b (and respectively,
// there should be enough space allocated in the solution vector x)
// returns true on success, false otherwise
static public native boolean SolveATA(int id, double[] b, double[] x); 

// Factors the A matrix alone. Assumes that A is square. If A is spd then Cholesky 
// factoring is applied; otherwise LU
static public native boolean FactorA(int id);

// Solve the square system Ax = b
// It is assumed that A is square and invertible;
static public native boolean SolveA(int id, double[] b, double[] x); 

// The version of SolveATA with 3 right-hand sides
// returns true on success, false otherwise
static public native boolean SolveATA3(int id, double[] bx, double[] by, double[] bz,
								   double[] x,        double[] y,        double[] z);

// The version of SolveATA with 2 right-hand sides
// returns true on success, false otherwise
static public native boolean SolveATA2(int id, double[] bx, double[] by,
								   double[] x,        double[] y);

// Will create *symbolic* factorization of ATA and store it aside
// returns the id of the factor stored, to be used later to factor; -1 if failed
// other matrices with the same zero structure
static public native int SymbolicFactorATA(int matrixId);

// Will create the actual numerical factorization of the given ATA matrix
// provided the symbolic factor with id factorId
static public native boolean FactorATA_UseSymbolic(int matrixID, int symbFactorId);

// Will solve the LS system using the precomputed factor
static public native boolean SolveATA_UseFactor(int matrixId, int factorId, double[] b, double[] x); 

// will free the factor
static public native void ReleaseFactor(int factorId);



								   
    static 
	{
        System.loadLibrary("taucs");
        InitTaucsInterface();
    }
    
    
    
    
    
    
    public static void main(String[] args) {
 
		//Taucs taucs = new Taucs();
		
	//Taucs.InitTaucsInterface();

	int idA, idB;

	idA = Taucs.CreateMatrix(4, 3);
	idB = Taucs.CreateMatrix(3, 2);

	Taucs.SetMatrixEntry(idA, 0, 0, 1);
	Taucs.SetMatrixEntry(idA, 1, 1, 2);
	Taucs.SetMatrixEntry(idA, 2, 2, 3);
	Taucs.SetMatrixEntry(idA, 3, 0, 1);

	Taucs.SetMatrixEntry(idB, 0, 0, -1);
	Taucs.SetMatrixEntry(idB, 1, 1, -2);
	Taucs.SetMatrixEntry(idB, 2, 0, 1);

	Taucs.FactorATA(idA);
	Taucs.FactorATA(idB);

	double[] b1 = {3, 4, 5, 6};
	double[] b2 = {3, 4, 5};

	double[] x1 = new double[3];
	double[] x2 = new double[3];

	Taucs.SolveATA(idA, b1, x1);
	Taucs.SolveATA(idB, b2, x2);


	for(int i=0; i<3; i++)
		System.out.print(" "+x1[i]);
	System.out.println();
	
	for(int i=0; i<3; i++)
		System.out.print(" "+x2[i]);
	System.out.println();

	Taucs.ReleaseMatrix(idA);
	Taucs.ReleaseMatrix(idB);
	
	Taucs.DeinitTaucsInterface();		

		
	}
}
