package umfpack;

import java.util.*;

public class SparseMatrix
{
	ArrayList[] columns;
	int n_columns;
	int n_rows;
	public SparseMatrix(int n_columns, int n_rows){
		this.n_columns = n_columns;
		this.n_rows = n_rows;
		columns = new ArrayList[n_columns];
		for(int i=0; i<n_columns; i++)
			columns[i] = new ArrayList();
	}
	public void add(int column_number, int row_number, double value){
		ArrayList column = (ArrayList) columns[column_number];
		
		for(int i=0; i<column.size(); i++){
			Cell cell = (Cell) column.get(i);
			if (cell.index == row_number){
				cell.value += value;
				return;
			}
			else if (cell.index > row_number) {
				cell = new Cell(row_number, value);
				column.add(i, cell);
				return;
			}
		}
		Cell cell = new Cell(row_number, value);
		column.add(cell);
		
	}
	
	public static SparseMatrix transpose(SparseMatrix matrix){
		SparseMatrix new_matrix = new SparseMatrix(matrix.n_rows, matrix.n_columns);
		for(int i=0; i<matrix.n_columns; i++){
			ArrayList column = matrix.columns[i];
			for(int j=0; j<column.size(); j++){
				Cell cell = (Cell) column.get(j);
			
				Cell new_cell = new Cell(i, cell.value);
				new_matrix.columns[cell.index].add(new_cell);
			}
		}
		return new_matrix;
	}
	public static double[] multiply(SparseMatrix matrix, double[] vector){
		double[] result = new double[matrix.n_columns];
		for(int i=0; i<matrix.n_columns; i++){
			ArrayList column = matrix.columns[i];
			for(int j=0; j<column.size(); j++){
				Cell cell = (Cell) column.get(j);
				result[i] += cell.value * vector[cell.index];
			}
		}
		return result;
	}
		
	public static SparseMatrix tranpose_and_multiply(SparseMatrix matrix){
		matrix = transpose(matrix);
		
		SparseMatrix new_matrix = new SparseMatrix(matrix.n_columns, matrix.n_columns);
		
		for(int i=0; i<new_matrix.n_columns; i++){
			new_matrix.columns[i] = new ArrayList();
			for(int j=0; j<new_matrix.n_columns; j++){
				double value = multiply(matrix.columns[i], matrix.columns[j], matrix.n_rows);
				if (value != 0)
					new_matrix.columns[i].add(new Cell(j, value));
			}
		}		
		return new_matrix;
	}
	
	


	static double value;
	static int index0, index1, n0, n1;
	static Cell cell0, cell1;
	public static double multiply(ArrayList column0, ArrayList column1, int n){
		
		value = 0;
		index0 = 1;
		index1 = 1;
		
		cell0 = (Cell)column0.get(0);
		cell1 = (Cell)column1.get(0);
		
		n0 = column0.size();
		n1 = column1.size();
			
		while(true){
			while(cell0.index < cell1.index){
				if (index0 == n0)	return value;
				cell0 = (Cell)column0.get(index0++);
			}
			while(cell1.index < cell0.index){
				if (index1 == n1)	return value;
				cell1 = (Cell)column1.get(index1++);
			}
			if (cell0.index == cell1.index){
				value += cell0.value * cell1.value;
				
				if (index0 == n0)	return value;
				cell0 = (Cell)column0.get(index0++);
				
				if (index1 == n1)	return value;
				cell1 = (Cell)column1.get(index1++);
			}
		}
	}

	public static int factorize(umfpack.Umfpack umfpack, SparseMatrix A){
    	int n = A.n_columns;
    	
    	int[] Ap = new int[n+1];
    	Vector _Ai = new Vector();
    	Vector _Ax = new Vector();
    	
  		for(int j=0; j<n; j++){
  			Ap[j] = _Ai.size();
			for(int i=0; i<A.columns[j].size(); i++){
				Cell cell = (Cell) A.columns[j].get(i);
				_Ai.addElement(new Integer( cell.index ));
				_Ax.addElement(new Double( cell.value ));
			}
    	}
  		Ap[n] = _Ai.size();
    	
    
		
    	int[] Ai = new int[_Ai.size()];
		for(int i=0; i<_Ai.size(); i++){
    		Ai[i] = ((Integer)_Ai.elementAt(i)).intValue();
		}
    		
    	double[] Ax = new double[_Ax.size()];
		for(int i=0; i<_Ax.size(); i++){
    		Ax[i] = ((Double)_Ax.elementAt(i)).doubleValue();
			//System.out.print(" "+Ax[i]);
		}
    		
 
    	return umfpack.factorize(n, Ap, Ai, Ax);
	}

}

	
	class Cell {
		int index;
		double value;
		Cell(int _index, double _value){
			index = _index;
			value = _value;
		}
	}